var canvas, stage;
var drawingCanvas;
var points = [];
var durationOfPoint = 15; // point exists for 15 frames
var circles = [];
var deviceOrientation = {x: 0, y: 0, z: 0};
var deviceHeading = 180;
var frame = 0;

function main(){
  initCanvas();
  initSensors();
  initSounds();
  var loading = document.getElementById("loading");
  loading.parentNode.removeChild(loading);
  applican.device.keepScreenOn(true);
}

function initCanvas() {
	canvas = document.getElementById("mainCanvas");
  var w = window,
    d = document,
    e = d.documentElement,
    g = d.getElementsByTagName('body')[0];
  canvas.width = w.innerWidth || e.clientWidth || g.clientWidth,
  canvas.height = w.innerHeight|| e.clientHeight|| g.clientHeight;

	stage = new createjs.Stage(canvas);
	stage.autoClear = false;
	stage.enableDOMEvents(true);

	createjs.Touch.enable(stage);
	createjs.Ticker.setFPS(30);
	createjs.Ticker.addEventListener("tick", handleTick);

	drawingCanvas = new createjs.Shape();
	stage.addEventListener("stagemousedown", handleMouseDown);
	stage.addEventListener("stagemouseup", handleMouseUp);

	stage.addChild(drawingCanvas);
	stage.update();
}

function initSensors() {
	applican.compass.watchHeading(watchHeadingSuccess, watchHeadingError, {frequency: 100});
	applican.accelerometer.watchAcceleration(watchAccelerationSuccess, watchAccelerationError, {frequency: 100});
	var options = {maximumAge: 0, enableHighAccuracy: false, frequency: 3000};
	applican.geolocation.watchPosition(watchPositionSuccess, watchPositionError, {enableHighAccuracy: false, frequency: 3000});
  applican.accelerometer.watchShake(shakeSuccess);
}

function handleMouseDown(event) {
  stage.addEventListener("stagemousemove" , handleMouseMove);
  applican.gamesound.playSE(2);
	points.push({point: {x: stage.mouseX, y: stage.mouseY}, frame: frame});
}

function handleMouseMove(event) {
  applican.gamesound.playSE(0);
	points.push({point: {x: stage.mouseX, y: stage.mouseY}, frame: frame});
}

function handleMouseUp(event) {
  applican.gamesound.playSE(2);
  stage.removeEventListener("stagemousemove" , handleMouseMove);
}

function handleTick() {
	frame++;
  var graphics = drawingCanvas.graphics.clear();
  drawBackground(graphics);
  handleCircles(graphics);
  handleLines(graphics);
  // TODO: handle frame overflow
  stage.update();
}

function drawBackground(graphics) {
  // calc base background color
  var r = 0.1 + deviceOrientation.z * 0.1;
  var g = 0.1 + deviceOrientation.x * 0.1;
  var b = 0.1 + deviceOrientation.y * 0.1;

  // calc gradient by calculating intersection with y=ax+d and y-axis, x-axis, y=height, x=width
  var a = deviceHeading / 360 * 3.14 * 3;
  if (a == 0) {
    graphics.beginFill(colorRgbText(r * 255, g * 255, b * 255)).drawRect(0, 0, width, height);
    return; // just giving up calculation for a moment. TODO: handle this
  }
  var height = stage.canvas.height;
  var width = stage.canvas.width;
  var d = height / width * a;
  var points = [{x: 0, y: d}, {x: width, y: a * width + d}, {x: -d / a, y: 0}, {x: (height - d) / a, y: height}];
  points = points.filter(function(p) {
    return 0 <= p.x && p.x <= width && 0<= p.y && p.y <= height;
  });
  if (points.length < 2) {
    graphics.beginFill(colorRgbText(r * 255, g * 255, b * 255)).drawRect(0, 0, width, height);
    return;
  }

  graphics.beginLinearGradientFill([colorRgbText(r * 255, g * 255, b * 255), "#222"], [0, 1], points[0].x, points[0].y, points[1].x, points[1].y).drawRect(0, 0, width, height);
}

function handleCircles(graphics) {
  if (Math.random() < 0.10) {
		addNewCircle();
	}
  for (var i = 0; i < circles.length; i++) {
    var c = circles[i];
    var c = circles[i];
		c.age++;
		var r = c.maxRadius * c.age / c.duration;
		graphics.beginRadialGradientFill([c.color, "#222"], [0.2, 1], c.center.x, c.center.y, 0, c.center.x, c.center.y, r).drawCircle(c.center.x, c.center.y, r);
	}

  // remove old items
	circles = circles.filter(function(c) {
		return c.age < c.duration;
	});
}

function addNewCircle() {
	circles.push({
		  center: {x: Math.random() * stage.canvas.width, y: Math.random() * stage.canvas.height},
	    maxRadius: Math.random() * 40 + 25,
			color: colorRgbText(Math.random() * 255, Math.random() * 255, Math.random() * 255),
			age: 0,
			duration: Math.random() * 15 + 5});
  applican.gamesound.playSE(1);
}

function handleLines(graphics) {
	for (var i = 1; i < points.length; i++) {
		var point = points[i]['point'];
		var lastPoint = points[i - 1]['point'];
		var center = {x: stage.canvas.width / 2, y: stage.canvas.height};
		var r = distance(point, lastPoint) / distance({x: 0, y: 0}, center) * 255.0 / 4;
		if (r > 255) { r = 255 }
		var g = point.y / stage.canvas.height * 255.0;
		var b = distance(point, center) / distance({x: 0, y: 0}, center) * 255.0;
		graphics.setStrokeStyle(1, 'round', 'round').beginStroke(colorRgbText(r, g, b)).moveTo(lastPoint.x, lastPoint.y).lineTo(point.x, point.y);
    graphics.moveTo(lastPoint.x, stage.canvas.height - lastPoint.y).lineTo(point.x, stage.canvas.height - point.y);
	}

	// remove old points
	var numOfShift = 0;
	for (var i = 0; i < points.length; i++) {
		if (points[i].frame + durationOfPoint < frame) {
			numOfShift++;
		} else {
			break;
		}
	}
	if (numOfShift > 0) {
		points = points.slice(numOfShift);
	}
}

function distance(p1, p2) {
	return Math.sqrt((p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y));
}

function colorRgbText(r, g, b) {
	return "rgb(" + Math.floor(r) + "," + Math.floor(g) + "," + Math.floor(b) + ")"
}
function watchHeadingSuccess(res){
  var deviceHeading = res.trueHeading;
}
function watchHeadingError(e){
  console.error("watchHeading error with code" + e.code);
}

function watchAccelerationSuccess(res){
  deviceOrientation = res;
}
function watchAccelerationError(){
	console.error("watchAccelerationError");
}

function watchPositionSuccess(res){
	var dump = "watchPositionSuccess\n";
	dump += "latitude:"+res.coords.latitude+"\n";
	dump += "longitude:"+res.coords.longitude+"\n";
	dump += "altitude:"+res.coords.altitude+"\n";
	dump += "accuracy:"+res.coords.accuracy+"\n";
	dump += "altitudeAccuracy:"+res.coords.altitudeAccuracy+"\n";
	dump += "heading:"+res.coords.heading+"\n";
	dump += "speed:"+res.coords.speed+"\n";
	dump += "timestamp:"+res.timestamp+"\n";
//	console.log(dump);
}

function watchPositionError(e){
	var dump = "watchPositionError\n";
	dump += "code:"+e.code+"\n";
	dump += "message:"+e.message+"\n";
  console.log(dump);
}

function shakeSuccess(){
  addNewCircle();
  setTimeout(function() {addNewCircle();}, 0.1)
}

function initSounds() {
  var bgmList = [
  	{track:0, src:"sound/contact04.mp3"}
  ];
  applican.gamesound.loadBGM(bgmList, loadBGMSuccess, loadBGMError);
  var seList = [
  	{track:0, src:"sound/contact02.mp3"},
  	{track:1, src:"sound/el-sound01.mp3"},
  	{track:2, src:"sound/contact5.mp3"}
  ];
  applican.gamesound.loadSE(seList, loadSESuccess, loadSEError);
}

function loadBGMSuccess() {
  // Test playing sound
  console.log("BGM should start");
  applican.gamesound.setBGMVolume(0, 50);
  applican.gamesound.playBGM({track: 0, loop: true});
}

function loadBGMError(err){
	console.error("fail to load bgm. Error code: " + err.code);
}

function loadSESuccess(){
  applican.gamesound.setBGMVolume(0, 80);
  applican.gamesound.setBGMVolume(1, 80);
  applican.gamesound.setBGMVolume(2, 80);
  console.log("SE should be played");
}

function loadSEError(err){
  console.error("fail to load SE. Error code: " + err.code);
}
